; ##############################################################################
; ##############################################################################
; ##############################################################################
; nur ein SLAVE am BUS, ROM + SCRATCHPAD auslesen und TEMPERATUREN berechnen	
;																				
; OUT: SRAM	(adr_TEMPERATURTEMPERATURWERTE)										
;																				
ONE_WIRE_AUSLESEN_SINGLE:
	; CLEAR										
	rcall CLEAR_TEMPERATURTEMPERATURWERTE	 
	rcall ONE_WIRE_CLEAR_ROM
	rcall ONE_WIRE_CLEAR_SCRATCHPAD
	; ROM (SINGLE)								
	rcall ONE_WIRE_SLAVE_SINGLE_ROM_READ		; OUT: (CARRY)				
	brcc ONE_WIRE_AUSLESEN_SINGLE_ERROR			; CARRY=0 ==> SPRUNG		
	; SCRATCHPAD+TEMPERATUR						
	rcall ONE_WIRE_SCRATCHPAD_TEMPERATUR		; OUT: CARRY		
	brcc ONE_WIRE_AUSLESEN_SINGLE_ERROR			; CARRY=0 ==> SPRUNG		
	; CARRY setzen								
	SEC	
	ret
;----------------------------------------------------------------------------------
ONE_WIRE_AUSLESEN_SINGLE_ERROR:
	; ERROR					
	ldi temp,(ERROR_SINGLE_MESSEN)	; CODE				
	rcall ERROR_SPEICHERN			; inp temp (CODE)	
	; CARRY clear			
	CLC
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; mehrere SLAVE am BUS, ein Slave direkt ansprechen, SCRATCHPAD auslesen 		
; und Temperatur berechnen														
;																				
; INP: SRAM (adr_ROM_CRC_BEREICH)												
; OUT: SRAM	(adr_TEMPERATURTEMPERATURWERTE)										
;																				
ONE_WIRE_AUSLESEN_DIREKT:
	; CLEAR						
	rcall ONE_WIRE_CLEAR_SCRATCHPAD
	; ROM CRC					
	rcall ONE_WIRE_ROM_CRC7					; OUT: CARRY		
	brcc ONE_WIRE_SLAVE_DIREKT_AUSLESEN_ERROR	; CARRY=0 ==> SPRUNG
	; SCRATCHPAD+TEMPERATUR		
	rcall ONE_WIRE_SCRATCHPAD_TEMPERATUR		; OUT: CARRY		
	brcc ONE_WIRE_SLAVE_DIREKT_AUSLESEN_ERROR	; CARRY=0 ==> SPRUNG
	; CARRY setzen				
	SEC	
	ret
;-------------------------------------------------------------------------------
ONE_WIRE_SLAVE_DIREKT_AUSLESEN_ERROR:
	; ERROR						
	ldi temp,(ERROR_DIREKT_MESSEN)	; CODE				
	rcall ERROR_SPEICHERN			; inp temp (CODE)	
	; CARRY clear				
	CLC
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; alle SLAVES aus der "LISTE" (SRAM)  auslesen und TEMPERATUREN berechnen		
;																				
; INP: SRAM (adr_ROM_SLAVE_LISTE)												
; OUT: SRAM	(adr_TEMPERATURTEMPERATURWERTE)										
;																				
ONE_WIRE_AUSLESEN_LISTE:
	; CLEAR				
	rcall CLEAR_TEMPERATURTEMPERATURWERTE	; 
	rcall ONE_WIRE_CLEAR_ROM
	rcall ONE_WIRE_CLEAR_SCRATCHPAD
	STS(adr_SLAVES_LFD_NR),NULL
	STS(adr_ANZAHL_SLAVES),NULL
;-------------------------------------------------------------------------------
ONE_WIRE_AUSLESEN_LISTE_NEXT:
	ldi ZL,  low(adr_ROM_LISTE)	    ; Z auf LISTE	
	ldi ZH, high(adr_ROM_LISTE)
	ldi XL,  low(adr_ROM_CRC_BEREICH)		; X auf ROM		
	ldi XH, high(adr_ROM_CRC_BEREICH)
	; lfd. NR Slaves		
	LDS temp,(adr_SLAVES_LFD_NR)
	; 8x addieren			
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	; KOPIE														
	LD temp1, Z+		; aus LISTE laden (adr_ROM_SLAVE_LISTE)	
	ST X+,temp1			; ablegen im SRAM (adr_ROM_CRC_BEREICH)	
	LD temp1, Z+		;
	ST X+,temp1			
	LD temp1, Z+		;
	ST X+,temp1			
	LD temp1, Z+		;
	ST X+,temp1			
	LD temp1, Z+		;
	ST X+,temp1			
	LD temp1, Z+		;
	ST X+,temp1			
	LD temp1, Z+		;
	ST X+,temp1			
	LD temp1, Z+		;
	ST X+,temp1			
	; ROM = 0 ??				
	LDS temp,(adr_ROM_CRC_BEREICH+0)
	tst temp
	breq ONE_WIRE_AUSLESEN_LISTE_FERTIG
	; ROM CRC					
	rcall ONE_WIRE_ROM_CRC7					; OUT: CARRY		
	brcc ONE_WIRE_AUSLESEN_LISTE_ERROR	; CARRY=0 ==> SPRUNG
	; MESSWERT auslesen			
	rcall ONE_WIRE_AUSLESEN_DIREKT			; OUT: CARRY		
	brcc ONE_WIRE_AUSLESEN_LISTE_ERROR	; CARRY=0 ==> SPRUNG
	; lfd. NR Slaves + 1		
	LDS temp,(adr_SLAVES_LFD_NR)
	inc temp
	STS(adr_SLAVES_LFD_NR),temp
	STS(adr_ANZAHL_SLAVES),temp
	rjmp ONE_WIRE_AUSLESEN_LISTE_NEXT
;-------------------------------------------------------------------------------
ONE_WIRE_AUSLESEN_LISTE_FERTIG:
	; CARRY setzen			
	SEC	
	ret
;-------------------------------------------------------------------------------
ONE_WIRE_AUSLESEN_LISTE_ERROR:
	; ERROR					
	ldi temp,(ERROR_LISTE_MESSEN)	; CODE				
	rcall ERROR_SPEICHERN			; inp temp (CODE)	
	; CARRY clear			
	CLC
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################









