; ##############################################################################
; ##############################################################################
; ##############################################################################
; INITIALISIERUNG																
;																				
; OUT: CARRY																	
;																				
ONE_WIRE_INITIALISIERUNG:
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; PULL-UP ein	
	rcall wait_5us							; kurze PAUSE	
	; PIN muss HIGH sein, bei LOW ev. Kurzschluss (oder anderes HARDWAREPROBLEM)
	sbis (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp ONE_WIRE_INITIALISIERUNG_ERROR
	; CARRY setzen			
	SEC
	ret
ONE_WIRE_INITIALISIERUNG_ERROR:
	; ERROR	
	ldi temp,(ERROR_INITIALISIERUNG)		; CODE				
	rcall ERROR_SPEICHERN					; inp temp (CODE)	
	; CARRY clear			
	CLC
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 1-Wire, Reset und Presence-Test												
;																				
; OUT: CARRY																	
;																				
ONE_WIRE_RESET_PRESENCE:
	; LED							
	sbi PORTD,LED_D_GRUEN					; LED-GRN	aus	
;-------------------------------------------------------------------------------
	; PIN muss HIGH sein, bei LOW ev. Kurzschluss (oder anderes HARDWAREPROBLEM)
	sbis (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp ONE_WIRE_RESET_PRESENCE_ERROR
;-------------------------------------------------------------------------------
	; Port auf Ausgang + LOW		
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL		
	; RESET IMPULS 480s (500 s)	
	rcall ONE_WIRE_WAIT_500us
	; PIN muss LOW sein, bei HIGH ev. Kurzschluss (oder anderes HARDWAREPROBLEM)
	sbic (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp ONE_WIRE_RESET_PRESENCE_ERROR
	; Port auf EINGANG + HIGH		
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	; Pause 70s (100s)			
	rcall ONE_WIRE_WAIT_100us
	; SAMPLES						
	sbic (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp ONE_WIRE_RESET_PRESENCE_ERROR
;-------------------------------------------------------------------------------
	cbi PORTD,LED_D_TRIGGER_SAMPLES_PRESENCE; TRIGGER		
	sbi PORTD,LED_D_TRIGGER_SAMPLES_PRESENCE; TRIGGER		
	cbi PORTD,LED_D_GRUEN					; LED-GRN an	
	rcall ONE_WIRE_WAIT_500us				; PAUSE			
	; vorsichtshalber den PIN auf AUSGANG (zur Vermeidung von Streinflssen)
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	; CARRY setzen			
	SEC
	ret
;-------------------------------------------------------------------------------
ONE_WIRE_RESET_PRESENCE_ERROR:
	; ERROR									
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; PULL-UP ein	

	ldi temp,(ERROR_PRESENCE)				; CODE				
	rcall ERROR_SPEICHERN					; inp temp (CODE)	
	; ANZEIGEN 
	rcall ANZEIGE_ERROR_CODE
	; CARRY clear			
	CLC
ret

; ##############################################################################
; ##############################################################################
; ##############################################################################
; 1 DATENBYTE 1-wire schreiben													
;																				
; Hier kommt es auf Zeiten an, stimmen die Zeiten nicht, werden die BITs vom	
; MASTER an den SLAVE nicht genau gelesen ! (ggf. mit OSZI prfen)				
;																				
; inp: temp1																	
;																				
; rcall DS_DATA_WRITE		; inp: temp1										
;																				
ONE_WIRE_DATA_WRITE:
	cbi PORTD,LED_D_TRIGGER_MASTER_TX	; TRIGGER		
	LSR temp1						; 1x rechts 1 rollt raus (LSB)
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-0 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-1 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-2 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-3 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-4 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-5 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-6 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus (MSB)
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-7 schreiben			
	; vorsichtshalber den PIN auf AUSGANG (zur Vermeidung von Streinflssen)
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	sbi PORTD,LED_D_TRIGGER_MASTER_TX 		; TRIGGER		
	; kurze Pause
	rcall ONE_WIRE_WAIT_500us	
ret
;-------------------------------------------------------------------------------
; DATEN-BIT schreiben															
; INP: CARRY																	
;																				
; rcall ONE_WIRE_DATA_WRITE_BIT		; INP: Carry								
;																				
ONE_WIRE_DATA_WRITE_BIT:
	; vorsichtshalber alle Interrupts sperren							
	CLI
	cbi PORTD,LED_D_GELB				; GELB 	
	; CARRY auswerten	
	brcs ONE_WIRE_DATA_WRITE_BIT_HIGH	; wenn CARRY=1 dann HIGH		
	rjmp ONE_WIRE_DATA_WRITE_BIT_LOW	; wenn CARRY=0 dann LOW 		
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_WRITE_BIT_LOW:
	; "0" SLOT 60...120s (LOW-PEGEL)									
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG				
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL					
	; Interrupts freigeben		
	SEI
	rcall ONE_WIRE_WAIT_100us				; 100s PAUSE = "0" SLOT	
	sbi PORTD,LED_D_GELB					; GELB 						
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 				

ret
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_WRITE_BIT_HIGH:
	; "1" SLOT 1...15s LOW_PEGEL, dann mind. 45s HIGH-PEGEL			
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG				
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL					
	rcall ONE_WIRE_WAIT_5us					; 5s PAUSE = "1" SLOT		
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 				
; Interrupts freigeben															
	SEI 									
	rcall ONE_WIRE_WAIT_100us				; 100s PAUSE = "1" SLOT	
	sbi PORTD,LED_D_GELB					; GELB 	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 1 DATENBYTE 1-wire lesen														
;																				
; Hier kommt es auf Zeiten an, stimmen die Zeiten nicht, werden die BITs vom	
; SLAVE nicht genau gelesen ! (ggf. mit OSZI prfen)							
;																				
; out: temp1																	
;																				
; rcall DS_DATA_READ		; OUT: temp1										
;																				
ONE_WIRE_DATA_READ:
	cbi PORTD,LED_D_TRIGGER_MASTER_RX; LED	
	cbi PORTD,LED_D_GELB			 ; GELB 	
	clr temp1

	rcall ONE_WIRE_DATA_READ_BIT	; LSB					
	ROR temp1						; 1x rechts ber CARRY 	
	rcall ONE_WIRE_DATA_READ_BIT
	ROR temp1						; 1x rechts ber CARRY 	
	rcall ONE_WIRE_DATA_READ_BIT
	ROR temp1						; 1x rechts ber CARRY 	
	rcall ONE_WIRE_DATA_READ_BIT
	ROR temp1						; 1x rechts ber CARRY 	
	rcall ONE_WIRE_DATA_READ_BIT
	ROR temp1						; 1x rechts ber CARRY 	
	rcall ONE_WIRE_DATA_READ_BIT
	ROR temp1						; 1x rechts ber CARRY 	
	rcall ONE_WIRE_DATA_READ_BIT
	ROR temp1						; 1x rechts ber CARRY 	
	rcall ONE_WIRE_DATA_READ_BIT
	ROR temp1						; 1x rechts ber CARRY 	
	; kurze Pause																
	rcall ONE_WIRE_WAIT_100us
	; vorsichtshalber den PIN auf AUSGANG+HIGH (zur Vermeidung von Streinflssen)
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	sbi PORTD,LED_D_TRIGGER_MASTER_RX; LED	
	sbi PORTD,LED_D_GELB			; GELB 	
	ret
;-------------------------------------------------------------------------------
; ein BIT vom SLAVE einlesen													
;																				
; OUT: CARRY																	
;																				
; rcall ONE_WIRE_DATA_READ_BIT		; OUT: CARRY 								
;																				
ONE_WIRE_DATA_READ_BIT:
	; vorsichtshalber alle Interrupts sperren					
	CLI
	; kurzer LOW-IMPULS 1...5s									
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL		
	; diese Pause ist kritisch, hier ggf. nachbessern		
	nop
	;rcall ONE_WIRE_WAIT_5us					; 5s PAUSE	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL (PULL-UP_EIN)	
	; diese Pause ist kritisch, hier ggf. nachbessern		
	rcall ONE_WIRE_WAIT_10us				; ca 10s warten
	cbi PORTD,LED_D_TRIGGER_SAMPLES_RX		; LED			
	; MASTER SAMPLES			
	sbic (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp ONE_WIRE_DATA_READ_1_SLOT
	rjmp ONE_WIRE_DATA_READ_0_SLOT
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_READ_0_SLOT:
	SEI 									; Interrupts freigeben		
	sbi PORTD,LED_D_TRIGGER_SAMPLES_RX		; LED			
	rcall ONE_WIRE_WAIT_100us				; warten		
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	CLC										; CARRY clear	
	ret
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_READ_1_SLOT:
	SEI 									; Interrupts freigeben		
	sbi PORTD,LED_D_TRIGGER_SAMPLES_RX		; LED			
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	rcall ONE_WIRE_WAIT_100us				; warten		
	SEC										; CARRY setzen	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; CRC Berechnung bernommen und abgendert von: http://s-huehn.de				
; CRC-Berechnung Application Note 27 von Dallas/Maxim							
;																				
; CRC-ROM																		
; INP: SRAM																		
; OUT: CARRY																	
;																				
ONE_WIRE_ROM_CRC7:
	push XL          ; auf dem Stack sichern            
	push XH
	push r16
	push r17
	push r18
	push r19
	push r20
	push r21
 	ldi XL,low (adr_ROM_CRC_BEREICH + 0) ;Zeiger 				
    ldi XH,high(adr_ROM_CRC_BEREICH + 0) 
    ldi     r19, 7                          ;7 Bytes berechnen			
    ldi     r20, 0x18                       ;Konstante fr Berechnung
    clr     r17                            ;CRC-Startwert = 0 setzen
ONE_WIRE_CRC7_100: 
	ld      r18, x+                         ;Byte aus Puffer holen
    mov     r21, r18                        ;Byte zwischenspeichern
    ldi     r16, 8                          ;8 Verschiebungen
ONE_WIRE_CRC7_200: 
	eor     r18, r17                      	;CRC berechnen
    ror     r18                             ;Ergebnis in Carry schieben
    mov     r18, r17                      	;letzten CRC-Wert holen
    brcc    ONE_WIRE_CRC7_300                          ;war Ergebnis = 0? ja -> weiter
    eor     r18, r20                        ;sonst CRC-Wert updaten
ONE_WIRE_CRC7_300: 
	ror     r18                             ;neuen CRC-Wert positionieren
    mov     r17, r18          	            ;und speichern
    mov     r18, r21                        ;restliche Bits holen
    bst     r18, 0                          ;LSB sichern, nchste Bitposition
    ror     r18                             ;gesichertes Bit als MSB einsetzen
    bld     r18, 7                          ;(Ersatz fr RR-Befehl)
    mov     r21, r18                        ;und Wert zwischenspeichern
    dec     r16                             ;alle Verschiebungen erledigt?
    brne    ONE_WIRE_CRC7_200               ;nein -> Schleife
    dec     r19                             ;alle Bytes bearbeitet?
    brne    ONE_WIRE_CRC7_100               ;nein -> Schleife
;-------------------------------------------------------------------------------
	LDS r16,(adr_ROM_CRC_BEREICH+7)
	cp r16,r17
	brne ONE_WIRE_CRC7_ERROR
	SEC										; CARRY=1	
	rjmp ONE_WIRE_CRC7_FERTIG
ONE_WIRE_CRC7_ERROR:
	; ERROR (kein speichern, nur LED) 					
	cbi PORTD,LED_D_ROT						; ROT		
	CLC										; CARRY=0	
ONE_WIRE_CRC7_FERTIG:
	pop r21      							; wiederherstellen                	
	pop r20
	pop r19
	pop r18
	pop r17
	pop r16
	pop XH
	pop XL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; CRC Berechnung bernommen und abgendert von: http://s-huehn.de				
; CRC-Berechnung Application Note 27 von Dallas/Maxim							
;																				
; CRC-SCRATCHPAD																
; INP: SRAM																		
; OUT: CARRY																	
;																				
ONE_WIRE_SCRATCHPAD_CRC8:
	push XL          ; auf dem Stack sichern            
	push XH
	push r16
	push r17
	push r18
	push r19
	push r20
	push r21
 	ldi XL,low (adr_SCRATCHPAD_CRC_BEREICH+0) ;Zeiger 				
    ldi XH,high(adr_SCRATCHPAD_CRC_BEREICH+0) 
    ldi     r19, 8                          ;8 Bytes berechnen			
    ldi     r20, 0x18                       ;Konstante fr Berechnung
    clr     r17                             ;CRC-Startwert = 0 setzen
ONE_WIRE_CRC8_100: 
	ld      r18, x+                         ;Byte aus Puffer holen
    mov     r21, r18                        ;Byte zwischenspeichern
    ldi     r16, 8                          ;8 Verschiebungen
ONE_WIRE_CRC8_200: 
	eor     r18, r17                       ;CRC berechnen
    ror     r18                             ;Ergebnis in Carry schieben
    mov     r18, r17                      ;letzten CRC-Wert holen
    brcc    ONE_WIRE_CRC8_300                          ;war Ergebnis = 0? ja -> weiter
    eor     r18, r20                        ;sonst CRC-Wert updaten
ONE_WIRE_CRC8_300: 
	ror     r18                             ;neuen CRC-Wert positionieren
    mov     r17, r18                      	;und speichern
    mov     r18, r21                        ;restliche Bits holen
    bst     r18, 0                          ;LSB sichern, nchste Bitposition
    ror     r18                             ;gesichertes Bit als MSB einsetzen
    bld     r18, 7                          ;(Ersatz fr RR-Befehl)
    mov     r21, r18                        ;und Wert zwischenspeichern
    dec     r16                             ;alle Verschiebungen erledigt?
    brne    ONE_WIRE_CRC8_200               ;nein -> Schleife
    dec     r19                             ;alle Bytes bearbeitet?
    brne    ONE_WIRE_CRC8_100               ;nein -> Schleife
;-------------------------------------------------------------------------------
	LDS r16,(adr_SCRATCHPAD_CRC_BEREICH+8)
	cp r16,r17
	brne ONE_WIRE_CRC8_ERROR
	SEC										; CARRY=1
	rjmp ONE_WIRE_CRC8_FERTIG
ONE_WIRE_CRC8_ERROR:
	; ERROR (kein speichern, nur LED) 					
	cbi PORTD,LED_D_ROT						; ROT	
	CLC										; CARRY=0
ONE_WIRE_CRC8_FERTIG:
	pop r21      							; wiederherstellen                	
	pop r20
	pop r19
	pop r18
	pop r17
	pop r16
	pop XH
	pop XL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ist SLAVE ein TEMPERATURSENSOR ?												
;																				
; INP: SRAM (adr_ROM_CRC_BEREICH+0) FAMILY CODE									
; OUT: CARRY																	
ONE_WIRE_CHECK_TEMPERATURSENSOR:
	; FAMILY CODE prfen, dann SCRATCHPAD
	LDS temp,(adr_ROM_CRC_BEREICH+0)			; FAMILY CODE	

	cpi temp, 0x10	; x10 DS1820 DS18s20 DS1920	
	breq ONE_WIRE_CHECK_TEMPERATURSENSOR_OK
	cpi temp, 0x22	; x22 DS1822				
	breq ONE_WIRE_CHECK_TEMPERATURSENSOR_OK
	cpi temp,0x28	; x28 DS18B20				
	breq ONE_WIRE_CHECK_TEMPERATURSENSOR_OK
	; CARRY clear								
	CLC
	ret
ONE_WIRE_CHECK_TEMPERATURSENSOR_OK:
	; CARRY setzen								
	SEC
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; LISTE (STAM) lschen															
ONE_WIRE_CLEAR_LISTE:
	ldi ZL, LOW (adr_ROM_LISTE)
	ldi ZH, HIGH(adr_ROM_LISTE)
	clr temp
ONE_WIRE_CLEAR_LISTE_s:
	ST z+,NULL
	dec temp
	brne ONE_WIRE_CLEAR_LISTE_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ONE_WIRE_CLEAR_ROM:
	STS(adr_ROM_CRC_BEREICH+0),NULL			; FAMILY	
	STS(adr_ROM_CRC_BEREICH+1),NULL
	STS(adr_ROM_CRC_BEREICH+2),NULL
	STS(adr_ROM_CRC_BEREICH+3),NULL
	STS(adr_ROM_CRC_BEREICH+4),NULL
	STS(adr_ROM_CRC_BEREICH+5),NULL
	STS(adr_ROM_CRC_BEREICH+6),NULL
	STS(adr_ROM_CRC_BEREICH+7),NULL 		; CRC		
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ONE_WIRE_CLEAR_SCRATCHPAD:
	STS(adr_SCRATCHPAD_CRC_BEREICH+0),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+1),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+2),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+3),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+4),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+5),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+6),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+7),NULL
	STS(adr_SCRATCHPAD_CRC_BEREICH+8),NULL ; CRC		
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 5s PAUSE																	
ONE_WIRE_WAIT_5us:
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 10s PAUSE																
ONE_WIRE_WAIT_10us:
	NOP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 100s PAUSE																
ONE_WIRE_WAIT_100us:
	push temp1          ; temp1 auf dem Stack sichern            
;------------
	ldi  temp1,(10*TAKT/1000000)
ONE_WIRE_WAIT_100us_w: 
    WDR					; WATCH-DOG		
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	dec  temp1
    brne ONE_WIRE_WAIT_100us_w
;------------
	pop temp1      		; temp1 wiederherstellen                    
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 500s PAUSE																
ONE_WIRE_WAIT_500us:
	push temp1          ; temp1 auf dem Stack sichern            
;------------
	ldi  temp1,125
wait_500us_w: 
    WDR					; WATCH-DOG		
	dec  temp1
    brne wait_500us_w
;------------
	pop temp1      		; temp1 wiederherstellen                    
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################


