; ##############################################################################
; ##############################################################################
; ##############################################################################
; SUCHE nach unbekannten SLAVES im BUS											
; REGISTER:																		
;	temp	   Hilfsregister													
;	temp1	(1)SUCHBYTE															
;	temp6	(6)LASTnull  (0...63 / 255 Sonderfunktion)							
;	temp9	(9)BITzhler (0...63)												
;																				
; 	OUT: CARRY; SRAM(adr_ANZAHL_SLAVES), SRAM (adr_ROM_SLAVE_LISTE) "LISTE"		
;																				
;-------------------------------------------------------------------------------
ROM_SEARCH:
	; Initialisierung										
	rcall ROM_SEARCH_KNOTEN_CLEAR		; KNOTEN clear		
	rcall ONE_WIRE_CLEAR_LISTE			; "LISTE" SRAM clear
	STS(adr_ANZAHL_SLAVES),NULL			; ANZ. SLAVES clear	

;===============================================================================
ROM_SEARCH_NEXT_SLAVE:
	;Reset und Presence-Test								
	rcall ONE_WIRE_RESET_PRESENCE		; OUT: CARRY		
	brcc ROM_SEARCH_ERROR				; CARRY=0 ? 		
	; MASTER TX xF0 (SEARCH ROM)							
	ldi	temp1,0xF0			
	rcall ONE_WIRE_DATA_WRITE
	; CLEAR HILFSREGISTER 									
	clr temp1							;(1)SUCHBYTE		
	ldi	temp6,255						;(6)LASTnull		
	clr temp9							;(9)BITzhler		
;-------------------------------------------------------------------------------
ROM_SEARCH_NEXT_BIT:
	; BIT-A lesen											
	clr temp
	rcall ONE_WIRE_DATA_READ_BIT		; OUT: CARRY		
	ROL temp							; 1xlinks ber C	
	; BIT-B lesen											
	rcall ONE_WIRE_DATA_READ_BIT		; OUT: CARRY		
	ROL temp							; 1xlinks ber C	
;-------------------------------------------------------------------------------
	; ERROR (A)=(B)=1 ?										
	cpi temp,0b00000011
	breq ROM_SEARCH_ERROR
;-------------------------------------------------------------------------------
	; vergleich temp1(A) temp2(B)							
	rcall ROM_SEARCH_AB_VERGLEICH 		; inp: temp			
;-------------------------------------------------------------------------------
	; ANZAHL SLAVES +1										
	tst temp9							;(9)BITZHLER		
	brne ROM_SEARCH_S1
	LDS temp,(adr_ANZAHL_SLAVES)
	inc temp
	STS(adr_ANZAHL_SLAVES),temp
ROM_SEARCH_S1:
	; ROM im SRAM ablegen									
	rcall ROM_SEARCH_SPEICHERN
;-------------------------------------------------------------------------------
	; BITZHLER + 1											
	inc temp9							;(9)BITZHLER		
	; alle Bits eingelesen ?								
	cpi temp9,63+1						;(9)BITZHLER		
	brne ROM_SEARCH_NEXT_BIT
;===============================================================================
	; KNOTEN setzen											
	rcall ROM_SEARCH_KNOTEN_setzen
	; CRC													
	rcall ONE_WIRE_ROM_CRC7				; OUT CARRY			
	brcc ROM_SEARCH_ERROR
	; max 30 SLAVES											
	LDS temp,(adr_ANZAHL_SLAVES)
	cpi temp,30
	brsh ROM_SEARCH_FERTIG
	; Schleife												
	cpi temp6,255						;(6)LASTnull		
	breq ROM_SEARCH_FERTIG
	rjmp ROM_SEARCH_NEXT_SLAVE
;===============================================================================
ROM_SEARCH_FERTIG:
	; CARRY setzen			
	SEC	
ret
;===============================================================================
ROM_SEARCH_ERROR:
	; ERROR																		
	ldi temp,(ERROR_ROM_SEARCH)				; CODE											
	rcall ERROR_SPEICHERN 					; OUT: CARRY=0 									
	; CARRY clear			
	CLC
	ret
; ##############################################################################
; inp: temp																		
; A=1 	0b00000010																
; B=1 	0b00000001																
; A=B=1 0b00000001																
;																				
; A und B miteinander vergleichen												
;																				
ROM_SEARCH_AB_VERGLEICH:
	; A=B=temp=0 ? 	(0=DISCREPANCY)								
	tst temp						; =0 ?						
	breq ROM_SEARCH_DISCREPANCY		; ja  =>SPRUNG				
;-------------------------------------------------------------------------------
ROM_SEARCH_NO_DISCREPANCY:	
	; (A) zurcksenden											
	andi temp,0b00000010			; (A) selektieren			
	tst temp						; (A)=0 ?					
	breq ROM_SEARCH_TX_NULL			; ja, A=0 ==> NULL senden	
	rjmp ROM_SEARCH_TX_EINS  		;     A=1 ==> EINS senden	
;===============================================================================
ROM_SEARCH_DISCREPANCY:
	; KNOTEN laden
	rcall ROM_SEARCH_KNOTEN_LADEN	; OUT: CARRY				
	brcs ROM_SEARCH_TX_EINS			; CARRY=1 ==> TX "1"		

	mov temp6,temp9					;(6)LASTnull (9)BITZHLER	
	rjmp ROM_SEARCH_TX_NULL			; CARRY=1 ==> TX "0"		

; ##############################################################################
; ##############################################################################
; ##############################################################################
; INP: temp9 (BITZHLER)														
; out: CARRY																	
ROM_SEARCH_KNOTEN_LADEN:
	; Z-initalisieren											
	ldi ZL,LOW (adr_ROM_SEARCH_KNOTEN)
	ldi ZH,HIGH(adr_ROM_SEARCH_KNOTEN)
	;(9)BITZHLER addieren										
	add ZL,temp9						;(9)BITZHLER			
	adc ZH,NULL
	LD temp,Z							; aus SRAM laden		
	tst temp							; ist NULL ?			
	brne ROM_SEARCH_KNOTEN_LADEN_EINS	; nein=>SPRUNG			
	CLC									; CARRY=0				
	ret
ROM_SEARCH_KNOTEN_LADEN_EINS:
	SEC									; CARRY=1				
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; die letzte NULLPOSITION auf eins setzen und den oberen Bereich lschen,		
ROM_SEARCH_KNOTEN_setzen:
	cpi temp6,255
	brne ROM_SEARCH_KNOTEN_setzen_w
	ret
ROM_SEARCH_KNOTEN_setzen_w:
	; Z-initalisieren											
	ldi ZL,LOW (adr_ROM_SEARCH_KNOTEN)
	ldi ZH,HIGH(adr_ROM_SEARCH_KNOTEN)
	; (6)LASTnull addieren										
	add ZL,temp6						;(6)LASTnull
	adc ZH,NULL
	; auf "1" und Z+1											
	ST Z+,EINS
	; Schleife													
	ldi temp,64+1
	sub temp,temp6						;(6)LASTnull
ROM_SEARCH_KNOTEN_setzen_s:
	ST z+,NULL
	dec temp
	brne ROM_SEARCH_KNOTEN_setzen_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ROM_SEARCH_KNOTEN_CLEAR:
	; Z-initalisieren				
	ldi ZL,LOW (adr_ROM_SEARCH_KNOTEN)
	ldi ZH,HIGH(adr_ROM_SEARCH_KNOTEN)
	; 64 Bytes CLEAR				
	ldi temp,65
ROM_SEARCH_KNOTEN_CLEAR_s:
	ST z+,NULL
	dec temp
	brne ROM_SEARCH_KNOTEN_CLEAR_s

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; WRITE BIT	0																	
ROM_SEARCH_TX_NULL:
	CLC								; CARRY=0	
	rcall ONE_WIRE_DATA_WRITE_BIT	; INP: Carry
	CLC								; CARRY=0	
	ROR temp1						; CARRY in's (1)SUCHBYTE	
	ret
;-------------------------------------------------------------------------------
; WRITE BIT	1																	
ROM_SEARCH_TX_EINS:
	SEC								; CARRY=1	
	rcall ONE_WIRE_DATA_WRITE_BIT	; INP: Carry
	SEC								; CARRY=1	
	ROR temp1						; CARRY in's (1)SUCHBYTE	
	ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ROM wird im SRAM abgelegt	fr sptere Auflistung und CRC						
; (adr_ROM_SLAVE_LISTE)															
; (adr_ROM_CRC_BEREICH)															
;																				
ROM_SEARCH_SPEICHERN:
	; SLAVE ANZAHL maximal 30	
	LDS temp,(adr_ANZAHL_SLAVES)
	cpi temp,30+1
	brlo ROM_SEARCH_SPEICHERN_w
	ret
ROM_SEARCH_SPEICHERN_w:
	; X-ZEIGER					
	ldi XL,LOW (adr_ROM_CRC_BEREICH)
	ldi XH,HIGH(adr_ROM_CRC_BEREICH)
	; Z-ZEIGER					
	ldi ZL,LOW (adr_ROM_LISTE)
	ldi ZH,HIGH(adr_ROM_LISTE)
	; -1						
	dec temp
	; 8 x addieren				
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	add ZL,temp
	adc ZH,NULL
	; Vergleich					
	cpi temp9,8-1
	breq ROM_SEARCH_SPEICHERN_8
	cpi temp9,16-1
	breq ROM_SEARCH_SPEICHERN_16
	cpi temp9,24-1
	breq ROM_SEARCH_SPEICHERN_24
	cpi temp9,32-1
	breq ROM_SEARCH_SPEICHERN_32
	cpi temp9,40-1
	breq ROM_SEARCH_SPEICHERN_40
	cpi temp9,48-1
	breq ROM_SEARCH_SPEICHERN_48
	cpi temp9,56-1
	breq ROM_SEARCH_SPEICHERN_56
	cpi temp9,64-1
	breq ROM_SEARCH_SPEICHERN_64
	ret
; SRAM Eintrag				
ROM_SEARCH_SPEICHERN_8:
	ST Z,temp1	; (adr_ROM_SLAVE_LISTE)
	ST X,temp1	; (adr_ROM_CRC_BEREICH)
	ret
ROM_SEARCH_SPEICHERN_16:
	adiw ZL,1
	ST Z,temp1
	adiw XL,1
	ST X,temp1
	ret
ROM_SEARCH_SPEICHERN_24:
	adiw ZL,2
	ST Z,temp1
	adiw XL,2
	ST X,temp1
	ret
ROM_SEARCH_SPEICHERN_32:
	adiw ZL,3
	ST Z,temp1
	adiw XL,3
	ST X,temp1
	ret
ROM_SEARCH_SPEICHERN_40:
	adiw ZL,4
	ST Z,temp1
	adiw XL,4
	ST X,temp1
	ret
ROM_SEARCH_SPEICHERN_48:
	adiw ZL,5
	ST Z,temp1
	adiw XL,5
	ST X,temp1
	ret
ROM_SEARCH_SPEICHERN_56:
	adiw ZL,6
	ST Z,temp1
	adiw XL,6
	ST X,temp1
	ret
ROM_SEARCH_SPEICHERN_64:
	adiw ZL,7
	ST Z,temp1
	adiw XL,7
	ST X,temp1
	ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

