;												
; Temperatursensoren auslesen					
; DS1820 DS18s20 DS18B20 DS1822 DS1920			
;												
; ATmega8 / 1MHz								
; Bernhard.Erfurt@gmx.de						
; 11/2011										
;												
; OLED/LCD 4x20									
;												
;																																	
; mgliche Varianten (Beispiele):																									
; - (SINGLE-MODUS) ROM eines unbekannten SLAVE auslesen und anzeigen																
; - (SINGLE-MODUS) SCRATCHPAD auslesen und Temperatur anzeigen																		
; - (MULTI-SLAVE) Einen SLAVE direkt ansprechen, nur ihn messen lassen, dann SCRATCHPAD auslesen und Temperaturen anzeigen			
; - (MULTI-SLAVE) Suche nach vorhandenen SLAVES, ist es ein Temperatursensor, dann SCRATCHPAD auslesen und Temperaturen anzeigen	
;																																	
; Der erste auftretende Fehler wird gespeichert und angezeigt																		
;																																	
; Im SRAM gibt es wichtige Bereiche:																								
;																																	
; - TEMPERATURTEMPERATURWERTE 	(Temperaturwerte der SLAVES)							
; - SCRATCHPAD_CRC				(jedes ausgelesene SCRATCHPAD wird hier abgelegt)		
; - ROM_CRC						(jede ROM-ID wird vorerst hier abgelegt und untersucht)	
; - ROM_LISTE					(eine Auflistung aller durch SEARCH gefundenen ROM-ID) kann aber auch durch die "Tabelle" gefllt werden
;												
;												
;												
;												
;												
;												
;												
;												
;												
;												
;												
;												
; Anschlussbelegung PINS						
;												
; PB0 ==> LCD									
; PB1 ==> LCD									
; PB2 ==> LCD									
; PB3 ==> LCD									
; PB4 ==> 										
; PB5 ==> 										
; PB6 ==> (Quarz)								
; PB7 ==> (Quarz)								
;												
; PC0 ==> 1-WIRE-DATENBUS						
; PC1 ==> 										
; PC2 ==> LCD									
; PC3 ==> LCD									
; PC4 ==> 										
; PC5 ==> 										
;												
; PD0 ==> (LED)		(SAMPLES RESET PRESENCE)	
; PD1 ==> (LED)		(MASTER TX)					
; PD2 ==> (LED)		(MASTER RX)					
; PD3 ==> (LED)		(MASTER SAMPLES_RX)			
; PD4 ==> (LED)									
; PD5 ==> LED GRN	(RESET PRESENCE OK)			
; PD6 ==> LED GELB	(MESSUNG/BUS aktiv)			
; PD7 ==> LED ROT	(ERROR)						
;												


; ATmega8 										
.include "m8def.inc"
; KONSTANTEN									
.equ TAKT					= 1000000			; 1 MHz	
; ERROR CODES									
.equ ERROR_TABELLE					=6	; *	mit TEXT
.equ ERROR_INITIALISIERUNG			=8
.equ ERROR_PRESENCE					=10	; * mit TEXT
.equ ERROR_MESSEN					=11
.equ ERROR_MESSEN_DIREKT			=12
.equ ERROR_SINGLE_MESSEN			=14
.equ ERROR_LISTE_MESSEN				=18
.equ ERROR_DIREKT_MESSEN			=19
.equ ERROR_SCRATCHPAD_READ			=20	; * mit TEXT
.equ ERROR_SINGLE_SCRATCHPAD_READ	=22
.equ ERROR_SINGLE_ROM_READ			=24
.equ ERROR_ROM_SEARCH				=28
.equ ERROR_TEMPERATURBERECHNUNG		=30 ; * mit TEXT
.equ ERROR_TEMPERATURBERECHNUNG_VA	=32
.equ ERROR_85_GRD					=85
; 1-WIRE-DATENBUS								
.equ ONE_WIRE_DDR			= DDRC
.equ ONE_WIRE_PORT			= PORTC
.equ ONE_WIRE_PIN			= PINC
.equ ONE_WIRE_PORTBIT		= 0
; LCD											
.equ LCD_DDR				= DDRB
.equ LCD_PORT				= PORTB
.equ LCD_D0					= 0
.equ LCD_D1					= 1
.equ LCD_D2					= 2
.equ LCD_D3					= 3
; LCD RS (Befehl/DATEN)							
.equ LCD_RS_DDR				= DDRC
.equ LCD_RS_PORT			= PORTC
.equ LCD_RS					= 2
; LCD_ENABLE									
.equ LCD_E_DDR				= DDRC
.equ LCD_E_PORT				= PORTC
.equ LCD_E					= 3
; LEDS 											
.equ LED_D_TRIGGER_SAMPLES_PRESENCE	= 0			; TRIGGER fr PRESENCE			
.equ LED_D_TRIGGER_MASTER_TX		= 1			; TRIGGER fr MASTER WRITE		
.equ LED_D_TRIGGER_MASTER_RX		= 2			; TRIGGER fr MASTER READ		
.equ LED_D_TRIGGER_SAMPLES_RX		= 3
.equ LED_D_RESERVE			= 4
.equ LED_D_GRUEN			= 5
.equ LED_D_GELB				= 6
.equ LED_D_ROT				= 7



; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN					= 0x0060		; 0x0060 START Adresse 		

.equ adr_ANZAHL_85GRD_MESSUNGEN	= 0x0060+6		; ANZAHL der ERRORS				

.equ adr_ANZAHL_SLAVES			= 0x0060+10		; ANZAHL gefundener SLAVES		

.equ adr_SLAVES_LFD_NR			= 0x0060+11		; lfd. Nr. des SLAVES			

.equ adr_ERROR_CODE				= 0x0060+20		; ERROR CODE vom ersten auftretenden Fehler		
.equ adr_ERROR_ROM_ID			= 0x0060+21		; 8BYTES, ROM-ID, vom ersten auftretenden Fehler

.equ adr_TEMPERATURTEMPERATURWERTE= 0x0060+100	; TEMPERATURWERTE LOW/HIGH (60BYTE)		

.equ adr_SCRATCHPAD_CRC_BEREICH	= 0x0060+400	; SCRATCHPAD+CRC(9BYTE)				
.equ adr_ROM_CRC_BEREICH		= 0x0060+450	; ROM + CRC		(8 BYTE)			
.equ adr_ROM_SEARCH_KNOTEN		= 0x0060+500	; fr ROM Suche (64Byte)			
.equ adr_ROM_LISTE				= 0x0060+600	; ROM			(8x300 BYTE=240Byte)

.equ adr_SRAM_CHECK				= 0x0060+943	; SRAM CHECK muss immer "S" sein	

; REGISTER										

.def DUMMY_R0	= R0	; wichtige Hilfsregister (nicht verwenden !)
.def DUMMY_R1	= R1	; wichtige Hilfsregister (nicht verwenden !)

.def NULL 		= R2
.def EINS 		= R3
.def VOLL		= R4

.def temp 		= R16
.def temp1 		= R17
.def temp2 		= R18
.def temp3 		= R19
.def temp4 		= R20
.def temp5 		= R21
.def temp6 		= R22
.def temp7 		= R23
.def temp8 		= R24
.def temp9 		= R25





.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   ; Timer0 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Bei unkorrekten Interrupt aufrufen erfolgt ein RESET ber WDR	
INTERRUPT_ERROR:
	wdr	
	cbi PORTD,LED_D_ROT		; LED ROT			
INTERRUPT_ERROR_SCHLEIFE:
	rjmp INTERRUPT_ERROR_SCHLEIFE

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM lschen										
SRAM_CLEAR:
	ldi ZL, LOW (adr_DATEN)
	ldi ZH, HIGH(adr_DATEN)
	clr temp
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND); Vergleich?		
	brlo SRAM_CLEAR_w	; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND)	; Vergleich?		
	brlo SRAM_CLEAR_w	; kleiner => SPRUNG	
	breq SRAM_CLEAR_w	; gleich  => SPRUNG	


; PORT B initialisieren								
	ldi temp, 0b00000000
	out DDRB, temp
; PORT B setzen										
	ldi temp, 0b00000000	
	out PORTB, temp				
; PORT C initialisieren								
	ldi temp, 0b00000000	
	out DDRC, temp
; PORT C setzen										
	ldi temp, 0b00000000	
	out PORTC, temp				
; PORT D initialisieren								
	ldi temp, 0b11111111	; LED-ROT/LED-GELB/LED-GRN/LED/LED/LED/LED/LED
	out DDRD, temp
; PORT D setzen										
	ldi temp, 0b00000000	; alle LEDs an			
	out PORTD, temp				
; WDR-INITIALISIERUNG								
	rcall WDR_INITIALISIERUNG
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; LEDs alle aus										
	sbi PORTD,LED_D_TRIGGER_SAMPLES_PRESENCE; LED	
	sbi PORTD,LED_D_TRIGGER_MASTER_TX		; LED	
	sbi PORTD,LED_D_TRIGGER_MASTER_RX		; LED	
	sbi PORTD,LED_D_TRIGGER_SAMPLES_RX		; LED	
	sbi PORTD,LED_D_RESERVE					; LED	
	sbi PORTD,LED_D_GRUEN					; GRN	
	sbi PORTD,LED_D_GELB					; GELB 	
	sbi PORTD,LED_D_ROT						; ROT	
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; VOLL				
	ldi temp,255
	mov VOLL,temp
	; ERROR CODE		
	STS(adr_ERROR_CODE),NULL
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
; LCD_INITIALISIERUNG								
	rcall LCD_INITIALISIERUNG
; ANZEIGE_BEGRUESSUNG								
	rcall ANZEIGE_BEGRUESSUNG
; ONE_WIRE_INITIALISIERUNG							
	rcall ONE_WIRE_INITIALISIERUNG
; CHECKS											
	rcall CHECK_TABELLE
	rcall PROGRAMM_CHECK
	rcall ANZEIGE_ERROR_CODE
; Interrupts freigeben								
	sei
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	; CLEAR					
	STS(adr_SLAVES_LFD_NR),NULL				; 
	STS(adr_ANZAHL_SLAVES),NULL
	rcall CLEAR_TEMPERATURTEMPERATURWERTE	; 
	rcall ONE_WIRE_CLEAR_LISTE
	rcall ONE_WIRE_CLEAR_ROM
	rcall ONE_WIRE_CLEAR_SCRATCHPAD
;-------------------------------------------------------------------------------
	;rcall BEISPIEL_1	; nur ein Slave vorhanden nur ROM auslesen und anzeigen												
	;rcall BEISPIEL_1A	; nur ein Slave vorhanden ROM und SCRATCHPAD auslesen und Temperatur anzeigen						
	;rcall BEISPIEL_2	; mehrere SLAVES vorhanden, ein Slave direkt ansprechen, SCRATCHPAD auslesen und Temperatur anzeigen
	;rcall BEISPIEL_3	; Slaves aus TABELLE abfragen, Temperaturen auslesen und anzeigen									
	rcall BEISPIEL_4	; nach SLAVES suchen, FAMILY und ROM und SCRATCHPAD anzeigen										
;-------------------------------------------------------------------------------
	rcall ANZEIGE_ERROR_CODE
	rcall PROGRAMM_CHECK
	rcall wait_100ms
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; nur ein Slave vorhanden ROM auslesen und anzeigen								
BEISPIEL_1:
	rcall ONE_WIRE_SLAVE_SINGLE_ROM_READ
	rcall ANZEIGE_SINGLE_FAMILY_ROM
	rcall wait_1s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; nur ein Slave vorhanden SCRATCHPAD und Temperatur anzeigen					
BEISPIEL_1A:
	rcall ONE_WIRE_MESSUNG_ALLE
	rcall ONE_WIRE_AUSLESEN_SINGLE
	rcall ANZEIGE_SINGLE_FAMILY_ROM_SCRATCHPAD_TEMPERATUR
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; mehrere SLAVES vorhanden, ein Slave direkt ansprechen SCRATCHPAD auslesen und Temperatur anzeigen
BEISPIEL_2:
	; ROM-ID in den SRAM kopieren	
	;0x0A00080253640910				
	ldi temp7, 0x0A
	ldi temp6, 0x00
	ldi temp5, 0x08
	ldi temp4, 0x02
	ldi temp3, 0x53
	ldi temp2, 0x64
	ldi temp1, 0x09
	ldi temp , 0x10
	STS(adr_ROM_CRC_BEREICH+7),temp7
	STS(adr_ROM_CRC_BEREICH+6),temp6
	STS(adr_ROM_CRC_BEREICH+5),temp5
	STS(adr_ROM_CRC_BEREICH+4),temp4
	STS(adr_ROM_CRC_BEREICH+3),temp3
	STS(adr_ROM_CRC_BEREICH+2),temp2
	STS(adr_ROM_CRC_BEREICH+1),temp1
	STS(adr_ROM_CRC_BEREICH+0),temp
	;rcall ONE_WIRE_MESSUNG_ALLE		; alle messen 			
	rcall ONE_WIRE_MESSEN_DIREKT	; nur ein SLAVE messen	
	rcall ONE_WIRE_AUSLESEN_DIREKT
	rcall ANZEIGE_SINGLE_FAMILY_ROM_SCRATCHPAD_TEMPERATUR 		; FAMILY+ROM+SCRATCHPAD anzeigen
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Slaves aus TABELLE abfragen, Temperaturen auslesen und anzeigen				
BEISPIEL_3:
	rcall ANZEIGE_WAIT									; wait...				
	rcall ONE_WIRE_MESSUNG_ALLE

	rcall TABELLE_NACH_LISTE_KOPIEREN
	rcall ONE_WIRE_AUSLESEN_LISTE
	rcall ANZEIGE_ROM_SEARCH_NUR_FAMILY_30_SLAVES_HEX 	;  30 x FAMILY			
	rcall wait_1s
	rcall wait_1s
	rcall ANZEIGE_TEMPERATUR_16_SLAVES
	rcall wait_1s
	rcall wait_1s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; nach SLAVES suchen, FAMILY und ROM und SCRATCHPAD anzeigen					
BEISPIEL_4:
	rcall ANZEIGE_WAIT									; wait...				
	rcall ONE_WIRE_MESSUNG_ALLE
	rcall LCD_PUNKT
	rcall ROM_SEARCH		
	rcall LCD_PUNKT
	rcall ONE_WIRE_AUSLESEN_LISTE
	rcall ANZEIGE_ERROR_CODE
	rcall ANZEIGE_ROM_SEARCH_NUR_FAMILY_30_SLAVES_HEX 	;  30 x FAMILY			
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall ANZEIGE_ROM_SEARCH_NUR_PS_30_SLAVES_HEX 		;  30 x Prfsumme		
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall ANZEIGE_TEMPERATUR_16_SLAVES
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s
	rcall ANZEIGE_ROM_SEARCH_FAMILY_ROM_SCRATCHPAD 		; FAMILY+ROM+SCRATCHPAD nacheinander anzeigen
	rcall wait_1s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; DATEIEN																		
.include "DS_LCD.asm"						; LCD allgemein										
.include "DS_LCD_OLED_4x20.asm"				; OLED 4x20											
.include "DS_SONSTIGES.asm" 				; WDR; PROGRAMMCHECK								
.include "DS_WAIT.asm"						; Warteschleifen									
.include "DS_1WIRE_BUS.asm"					; 1-WIRE-BUS RESET,RX,TX,CRC						
.include "DS_1WIRE_ROM_SEARCH.asm"			; 1-WIRE-SEARCH										
.include "DS_1WIRE_MESSUNG.asm"				; 1-WIRE-MESSEN										
.include "DS_1WIRE_AUSLESEN.asm"			; 1-WIRE-AUSLESEN MESSWERT			
.include "DS_1WIRE_SINGLE.asm"				; 1-WIRE nur ein SLAVE vorhanden	
.include "DS_1WIRE_SCRATCHPAD_TEMPERATUR.asm"; TEMPERATURBERECHNUNG				
; ##############################################################################
; ##############################################################################
; ##############################################################################
; TEXTE																			
TEXT_BEGRUESSUNG_1:
.db "DS1820 DS18s20",0
TEXT_BEGRUESSUNG_2:
.db "1-WIRE",0
TEXT_BEGRUESSUNG_3:
.db "ATmega8        1 MHz",0
TEXT_BEGRUESSUNG_4:
.db "10.11.2011 - V6a",0
;-------------------------------------------------------------------------------
TEXT_ERROR_UNDEFINIERT:
.db "SIEHE PROGRAMMCODE !",0
TEXT_CODE_06:
.db "TABELLE korrekt ?",0
TEXT_CODE_10:
.db "HARDWAREPROBLEM ?",0
TEXT_CODE_20:
.db "SCRATCHPAD-READ",0
TEXT_CODE_30:
.db "TEMPERATURBERECHNUNG",0
TEXT_CODE_85:
.db "SENSOR no MESSUNG",0
;---- FAMILY CODES -------------------------------------------------------------
TEXT_FAMILY_x05:
.db "DS2405",0
TEXT_FAMILY_x09:
.db "DS2502",0
TEXT_FAMILY_x0B:
.db "DS1985",0
TEXT_FAMILY_x10:
.db "DS18s20 DS1920",0
TEXT_FAMILY_x12:
.db "DS2406 DS 2407",0
TEXT_FAMILY_x14:
.db "DS2430A EEPROM 256",0
TEXT_FAMILY_x1C:
.db "DS2422",0
TEXT_FAMILY_x1D:
.db "DS2423",0
TEXT_FAMILY_x1F:
.db "DS2409",0
TEXT_FAMILY_x22:
.db "DS1822",0
TEXT_FAMILY_x23:
.db "DS2433 EEPROM 4k",0
TEXT_FAMILY_x26:
.db "DS2438",0
TEXT_FAMILY_x27:
.db "DS2417 RTC",0
TEXT_FAMILY_x28:
.db "DS18B20",0
TEXT_FAMILY_x2C:
.db "DS2890",0
; ##############################################################################
; ##############################################################################
; ##############################################################################
SENSOR_ROM_00:
;   MSB(CRC)						LSB(FAMILY)
.db 0x0A,0x00,0x08,0x02,0x53,0x64,0x09,0x10
SENSOR_ROM_01:
.db 0x8B,0x00,0x08,0x02,0x53,0x66,0x1A,0x10
SENSOR_ROM_02:
.db 0x45,0x00,0x08,0x02,0x53,0x79,0x9B,0x10
SENSOR_ROM_03:
.db 0x0B,0x00,0x08,0x02,0x53,0x5F,0xED,0x10
SENSOR_ROM_04:
.db 0x58,0x00,0x08,0x02,0x53,0xAE,0x10,0x10
SENSOR_ROM_05:
.db 0x97,0x00,0x08,0x02,0x53,0xA2,0xB6,0x10
SENSOR_ROM_06:
.db 0xD3,0x00,0x08,0x02,0x53,0x66,0x9C,0x10
SENSOR_ROM_07:
.db 0xC4,0x00,0x08,0x02,0x53,0x94,0x3E,0x10
SENSOR_ROM_08:
.db 0x5F,0x00,0x08,0x02,0x53,0x7D,0x38,0x10
SENSOR_ROM_09:
.db 0xAE,0x00,0x08,0x02,0x53,0x80,0xF7,0x10
SENSOR_ROM_10:
.db 0x7A,0x00,0x00,0x00,0x29,0xDB,0x3A,0x22
SENSOR_ROM_11:
.db 0x26,0x00,0x00,0x00,0x29,0xF4,0x16,0x22
SENSOR_ROM_12:
.db 0x42,0x00,0x00,0x00,0x29,0xDE,0xBA,0x22
SENSOR_ROM_13:
.db 0x8C,0x00,0x00,0x00,0x27,0x62,0x0C,0x22
SENSOR_ROM_14:
.db 0xE6,0x00,0x00,0x00,0x29,0xDE,0x11,0x22
SENSOR_ROM_15:
.db 0xFD,0x00,0x00,0x00,0x29,0xE4,0x2B,0x22
SENSOR_ROM_16:
.db 0x91,0x00,0x00,0x00,0x29,0xF1,0xCE,0x22
SENSOR_ROM_17:
.db 0x7B,0x00,0x00,0x00,0x29,0xDC,0xF0,0x22
SENSOR_ROM_18:
.db 0x6B,0x00,0x00,0x00,0x29,0xFB,0x2C,0x22
SENSOR_ROM_19:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_20:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_21:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_22:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_23:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_24:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_25:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_26:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_27:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_28:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22
SENSOR_ROM_29:
.db 0xDA,0x00,0x00,0x00,0x29,0xEA,0x2C,0x22



















